DATASET_MESSAGE = 'Select dataset'

INFO_DATASET_PREFIX = 'Dataset info:\n\n'

NO_DATASET = 'No dataset selected'

INFO_DATASET_MESSAGE = INFO_DATASET_PREFIX + NO_DATASET + '\n\n'

PREDICTOR_MESSAGE = 'Select predictor'

INFO_PREDICTOR_PREFIX = 'Predictor info:\n'

INFO_PREDICTOR_MESSAGE = f'\n\n{INFO_PREDICTOR_PREFIX}\nNo predictor trained\n\n\n\n\n\n'

PREDICTOR_PERFORMANCE_PREFIX = 'Predictor performance:\n'

EXTRACTOR_MESSAGE = 'Select extractor'

INFO_EXTRACTOR_PREFIX = 'Extractor info:\n'

INFO_EXTRACTOR_MESSAGE = f'\n\n{INFO_EXTRACTOR_PREFIX}\nNo extractor trained\n\n\n\n\n\n'

THEORY_PREFIX = 'Extracted theory'

THEORY_MESSAGE = 'No extractor trained\n\n\n'

EXTRACTOR_PERFORMANCE_PREFIX = 'Extractor performance:\n'

THEORY_ERROR_MESSAGE = {
    'amount': 'Too many extracted rules\n\n\n',
    'length': 'Too long clauses\n\n\n',
}
